/*
 * grunt-plugin-build
 *
 *
 * Copyright (c) 2016 Jeroen de Lau
 * Licensed under the Proprietory license.
 */

'use strict';

var merge = require('merge');
var ManifestProcessor = require("./Builder/ManifestProcessor");
var Copier = require("./Builder/Copier");

module.exports = function (grunt) {

    grunt.PluginBuilder = grunt.PluginBuilder || {};

    function Builder(settings)
    {
        this.grunt = grunt;
        this.settings = settings;

        this.manifest = new ManifestProcessor(this, settings.manifestPath);
        merge(this.settings, this.manifest.buildInfo);

        this.copier = new Copier(grunt, this.getSrcPath(), this.getBuildPath());
    }

    var method = Builder.prototype;

    /**
     * Returns the where files will be copied
     *
     * @returns {String}
     */
    method.getBuildPath = function ()
    {
        var path = this.settings.buildPath;
        //path += this.settings.config ? '/' + this.settings.config : '';
        //path += "/" + this.settings.browser;
        path += this.settings.browserPath ? "/" + this.settings.browserPath : "";
        return path;
    };

    /**
     * Return where files should be copied from
     *
     * @returns {String}
     */
    method.getSrcPath = function ()
    {
        return this.settings.srcPath;
    };

    /**
     * Return the contents of the manifest file
     *
     * @returns {object}
     */
    method.getManifest = function ()
    {
        return grunt.file.readJSON(this.getSrcPath() + '/manifest.json');
    };


    /**
     * Copy required resources to build folder.
     *
     * Will replace all placeholders in the list with replacements
     *
     * If this.settings.onlyRqeuiredSource = true, it will only
     * copy the source that is indicated in the manifest and files
     *
     * @returns void
     */
    method.build = function (callback)
    {
        //Create the directory
        grunt.file.mkdir(this.getBuildPath());

        var sp = this.getSrcPath();
        var bp = this.getBuildPath();
        if (this.settings.omitUnused) {
            grunt.verbose.writeln('Copying REQUIRED files ' + sp + ' > ' + bp);
            this.manifest.aggregateFiles(this.copier, aggregateComplete.bind(this));
        } else {
            grunt.verbose.writeln('Copying ALL files ' + sp + ' > ' + bp);
            this.copier.addFolder("");

            //this._copyFolder(sp, bp);
            aggregateComplete.bind(this)();
            //callback(true);
        }

        function aggregateComplete()
        {
            this.manifest.write(bp);
            this.copier.copyFiles(this.settings.replacements);
            callback(true);
        }


    };

    grunt.PluginBuilder.Builder = Builder;
};
