/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

var merge = require('merge');

function Copier(grunt, srcPath, destPath)
{
    this.files = [];
    this.srcPath = srcPath;
    this.destPath = destPath;
    this.grunt = grunt;
}

var method = Copier.prototype;


method.addPath = function (path)
{
    if(path !== "")
    {
        this.files.push(path);
    }
};

method.addFolder = function (path)
{
    path = this.srcPath + "/" + path + "/**/*.*";
    var paths = this.grunt.file.expand(path);

    for (var i = 0; i < paths.length; i++)
    {
        var relPath = paths[i].substr(this.srcPath.length + 1, paths[i].length);
        this.addPath(relPath);
    }
};

/**
 * Copy required resources to build folder.
 *
 * Will replace all placeholders in the list with replacements
 *
 * If this.settings.onlyRqeuiredSource = true, it will only
 * copy the source that is indicated in the manifest and files
 *
 * @returns void
 */
method.copyFiles = function (replacements)
{
    this.grunt.log.writelns("Copying: " + this.files.length + " files");
    for( var i = 0 ; i < this.files.length ; i++ )
    {
        var src = this.srcPath + "/" + this.files[i];
        var dest = this.destPath + "/" + this.files[i];
        this._copyFile(src, dest, replacements);
        //console.log(this.destPath,  this.files[i]);
    }
};

/**
 * Copy a file, and replace placeholders
 *
 * @param {type} sourceFilePath
 * @param {type} destFilePath
 * @returns void
 */
method._copyFile = function (sourceFilePath, destFilePath, replacements)
{
    if (sourceFilePath.match(/(\.html|\.js|\.css|\.json)$/))
    {
        this.grunt.file.copy(sourceFilePath, destFilePath, {
            encoding: 'utf8',
            process: _replacePlaceholder
        });
    } else {
        this.grunt.file.copy(sourceFilePath, destFilePath);
    }


    function _replacePlaceholder(contents, sourceFilePath, destFilePath) {
        for (var placeholder in replacements)
        {
            var val = replacements[placeholder];
            var lookup = "@" + placeholder + "@";
            //if this is not a string, remove surrounding quotes from
            //the placeholder in the code
            if (typeof (val) !== 'string')
            {
                lookup = '(["\'])' + lookup + '\\1';
            }
            var regex = new RegExp(lookup, "g");
            console.log(sourceFilePath);
            contents = contents.replace(regex, replacements[placeholder]);
        }
        return contents;
    }
};



module.exports = Copier;
