/*
 * grunt-plugin-build
 *
 *
 * Copyright (c) 2016 Jeroen de Lau
 * Licensed under the Proprietory license.
 */

'use strict';

var path = require('path');
var fs = require('fs');
var cheerio = require('cheerio');
var ScriptProcessor = require("./ScriptProcessor");


function HtmlProcessor(builder, processors, path)
{
    this.processors = processors;
    this.grunt = builder.grunt;
    this.path = path;
    this.builder = builder;

    var absPath = builder.getSrcPath() + "/" + path;
    this.$ = cheerio.load(this.grunt.file.read(absPath), {lowerCaseAttributeNames: false});

    this.processors.push(new ScriptProcessor(this.builder, this.getScripts()));
}

var method = HtmlProcessor.prototype;


method.aggregateFiles = function (copier, callback)
{
    var cssPaths = this.getCssPaths();
    for(var i = 0 ; i < cssPaths.length ; i++ )
    {
        copier.addPath(this._normalizePath(cssPaths[i]));
    }

    callback();
};

method._normalizePath = function(path)
{
    var parts = this.path.split("/");
    parts.pop();
    var pathParts = path.split("/");

    for(var i = 0 ; i < pathParts.length ; i++ )
    {
        if(pathParts[i] === "..")
        {
            parts.pop();
        }else{
            parts.push(pathParts[i]);
        }
    }
    return parts.join("/");
};

method.getResourcePaths = function ()
{
    return this.getJsPaths().concat(this.getCssPaths());
};

method.getScripts = function()
{
    var paths = this.getJsPaths();
    for(var i = 0 ; i < paths.length ; i++)
    {
        paths[i] = this._normalizePath(paths[i]);
    }
    return paths;
};

/**
 * Returns array with js files in the html
 *
 * @returns {Array}
 */
method.getJsPaths = function ()
{
    var paths = [];
    var $ = this.$;
    $('script').each(function (i, element) {
        paths[i] = $(this).attr("src");
    });
    return paths;
};

/**
 * Returns array with css files in the html
 *
 * @returns {Array}
 */
method.getCssPaths = function ()
{
    var paths = [];
    var $ = this.$;
    $('link[rel=stylesheet]').each(function (i, elem) {
        paths[i] = $(this).attr("href");
    });
    return paths;
};

module.exports = HtmlProcessor;

