/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

var merge = require('merge');
var HtmlProcessor = require("./HtmlProcessor");
var ScriptProcessor = require("./ScriptProcessor");

function ManifestProcessor(builder, file)
{
    this.grunt = builder.grunt;
    this.builder = builder;
    this.buildInfo = {};
    this.data = null;
    this.replacements = builder.settings.replacements;

    this.processors = [];

    this._read(file);
    this._extend();

    if (this.data.build)
    {
        this.buildInfo = this.data.build;
        delete this.data.build;
    }
}

var method = ManifestProcessor.prototype;

method.write = function (buildPath)
{
    var path = buildPath + "/manifest.json";
    this.grunt.file.write(path, JSON.stringify(this.data, null, 2));
};

method.aggregateFiles = function (copier, callback)
{

    this._addIncludePaths(copier);
    this._addAssets(copier);
    this._addHtml(copier);
    this._addBackground(copier);
    this._addContentScripts(copier);
    this._addWebAccessibleResources(copier);

    var count = 0;
    var total = this.processors.length;

    for (var i = 0; i < this.processors.length; i++)
    {
        this.processors[i].aggregateFiles(copier, complete);
    }

    function complete()
    {
        count++;
        if (count === total)
        {
            callback();
        }
    }
};

method._addIncludePaths = function (copier)
{
    var paths = this.builder.settings.includePaths;
    if (paths) {
        for (var i = 0; i < paths.length; i++)
        {
            copier.addFolder(paths[i]);
        }
    }
};

method._addAssets = function (copier)
{
    copier.addFolder('assets');
};

method._addHtml = function (copier)
{
    var path = null;

    var popup = this.data.browser_action || this.data.page_action;
    if (popup && popup.default_popup)
    {
        path = popup.default_popup;
        copier.addPath(path);
        this.processors.push(new HtmlProcessor(this.builder, this.processors, path));
    }

    if (this.data.options_ui)
    {
        path = this.data.options_ui.page;
        copier.addPath(path);
        this.processors.push(new HtmlProcessor(this.builder, this.processors, path));
    }

    var include = this.buildInfo.include;
    if (include && include.html)
    {
        for (var i = 0; i < include.html.length; i++)
        {
            path = include.html[i];
            copier.addPath(path);
            this.processors.push(new HtmlProcessor(this.builder, this.processors, path));
        }
    }
}
;


method._addBackground = function (copier)
{
    if (this.data.background)
    {
        var scripts = this.data.background.scripts;
        this.processors.push(new ScriptProcessor(this.builder, scripts));
    }
};

method._addContentScripts = function (copier)
{
    if (this.data.content_scripts)
    {
        var targets = this.data.content_scripts;

        for (var i = 0; i < targets.length; i++)
        {
            var scripts = targets[i].js;
            if (scripts)
            {
                this.processors.push(new ScriptProcessor(this.builder, scripts));
            }

            var cssFiles = targets[i].css;
            for (var j = 0; j < cssFiles.length; j++)
            {
                copier.addPath(cssFiles[j]);
            }
        }
    }
};

method._addWebAccessibleResources = function (copier)
{
    if (this.data.web_accessible_resources)
    {
        var paths = this.data.web_accessible_resources;

        for (var i = 0; i < paths.length; i++)
        {
            if (paths[i].substr("*") < 0) {
                copier.addPath(paths[i]);
            } else {
                copier.addFolder(paths[i]);
            }
        }
    }
};

method._read = function (path)
{
    var text = this.grunt.file.read(this.builder.getSrcPath() + path);
    this.data = JSON.parse(this._replacePlaceholder(text));


};


method._replacePlaceholder = function (contents) {

    for (var placeholder in this.replacements)
    {
        var val = this.replacements[placeholder];
        var lookup = "@" + placeholder + "@";

        //if this is not a string, remove surrounding quotes
        if (typeof (val) !== 'string')
        {
            lookup = '(["\'])' + lookup + '\\1';
        }
        var regex = new RegExp(lookup, "g");

        contents = contents.replace(regex, this.replacements[placeholder]);
    }
    return contents;
};


/**
 * Extend the manifest with the provided config
 */
method._extend = function ()
{
    while (this.data.build && this.data.build.extends)
    {
        var path = this.data.build.extends;
        delete this.data.build.extends;
        var data = this.grunt.file.readJSON(this.builder.getSrcPath() + path);
        merge(this.data, data);
    }
};

module.exports = ManifestProcessor;

