/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */



var merge = require('merge');
var require = require("requirejs");

function ScriptProcessor(builder, scripts) {
    this.scripts = scripts;
    this.builder = builder;
    this.grunt = builder.grunt;
    this.mainFilePath = null;


    var requireScripts = this.isRequire();
    if (requireScripts)
    {
        this.config = this._parseConfig(requireScripts.config);
        this.main = this._getMain(requireScripts.main);
        this.mainFilePath = requireScripts.main;
    }

}

ScriptProcessor.cache = {};

var method = ScriptProcessor.prototype;

method.isRequire = function ()
{
    for (var i = 0; i < this.scripts.length; i++)
    {
        if (this.scripts[i].indexOf("require.js") >= 0)
        {
            return {
                'config': this.scripts[i + 1],
                'main': this.scripts[i + 2]
            };
        }
    }
    return false;
};

method.aggregateFiles = function (copier, callback)
{
    var scripts = this.scripts;
    var $this = this;
    if (this.config && this.main)
    {
        this.getRequiredFiles(addFiles);
    } else {
        addFiles.bind(this)();
    }

    function addFiles()
    {
        for (var i = 0; i < $this.scripts.length; i++)
        {
            copier.addPath($this.scripts[i]);
        }
        callback();
    }
};

method.getRequiredFiles = function (callback)
{

    var mainFilePath = this.mainFilePath;
    var file = this.builder.getBuildPath() + '/' + mainFilePath;
    var minify = this.builder.settings.minify;
    var useFileCache = this.builder.settings.useFileCache;
    var scripts = this.scripts;
    var srcPath = this.builder.getSrcPath();
    var grunt = this.grunt;
    var mangle = this.builder.settings.mangle;
    var $this = this;

    var cachekey = this.mainFilePath + mangle;
    var fileCachePath = "cache/" + cachekey + ".json";
    if (useFileCache && grunt.file.exists(fileCachePath))
    {
        var fileCache = grunt.file.readJSON("cache/" + cachekey + ".json");
        ScriptProcessor.cache[cachekey] = fileCache;
    }

    if (ScriptProcessor.cache[cachekey])
    {
        done();
    } else if (this.builder.settings.fastBuild)
    {
        //is this a fastbuild
    } else {
        var config = this.buildConfig(file, mangle);
        require.optimize(config, processed);
    }

    function processed(output)
    {
        var cache = {
            minified: [],
            regular: [],
            consolidated: null
        };

        for (var i = 0; i < scripts.length; i++) {
            if (scripts[i].indexOf(mainFilePath) < 0)
            {
                cache.minified.push(scripts[i]);
            }
            cache.regular.push(scripts[i]);
        }

        cache.consolidated = grunt.file.read(file);
        //console.log(output);
        var paths = output.split("\n");
        var base = paths[1].replace(file, "") + srcPath + "/";

        for (var i = 3; i < paths.length; i++) {
            //for now exclude the css

            if (paths[i].indexOf("css!") !== -1) {
                paths[i] = $this.normalizeCss(paths[i], config);
            }
            cache.regular.push(paths[i].replace(base, ""));
        }

        ScriptProcessor.cache[cachekey] = cache;
        grunt.file.write(fileCachePath, JSON.stringify(cache));
        done();
    }

    function done()
    {
        var cache = ScriptProcessor.cache[cachekey];
        $this.scripts = minify ? cache.minified : cache.regular;

        if (minify) {
            grunt.file.write(file, cache.consolidated);
        }

        try {
            callback();
        } catch (e) {
            grunt.log.errorlns(e.toString());
        }
    }

};

method.normalizeCss = function (path, config)
{
    var alias = path.split("!")[1];
    var index = alias.split("/")[0];
    var basePath = config.paths[index];
    var relPath = alias.split("/");
    relPath[0] = basePath;
    return relPath.join("/") + ".css";
};

method.buildConfig = function (outputPath, mangle)
{
    var buildConfig = {
        'name': this.main,
        'out': outputPath
    };

    if (mangle) {
        buildConfig.uglify2 = {
            'mangle': true,
            'mangleProperties': {
                'regex': /^_/
            }
        };
    }

    return merge(merge(true, this.config), buildConfig);
};

method._parseConfig = function (configPath)
{
    var file = this.grunt.file.read(this.builder.getSrcPath() + "/" + configPath);
    var configString = file.match(/config\(([\s\S]*?)\);/)[1];


    //console.log(configString);
    //can't use JSON.parse, since the config string is not always valid
    //json, so use eval instead;
    var config = null;
    eval("var config = " + configString + ";");
    //var config = JSON.parse(configString);

    //parse the paths in config so they work
    var src = this.builder.getSrcPath();

    config.baseUrl = src + config.baseUrl;
    for (var lib in config.paths)
    {
        if (config.paths[lib][0] === "/")
        {
            config.paths[lib] = "../" + config.paths[lib];
        }
    }
    return config;
};

method._getMain = function (mainFilePath)
{
    //console.log(mainFilePath.replace(".js", "").replace("js/", ""));
    return mainFilePath.replace(".js", "").replace("js/", "");
};

module.exports = ScriptProcessor;

